@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id$
+/*  $Id: panel-dialogs.c 26046 2007-09-05 11:54:36Z nick $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *  Copyright © 2006 Benedikt Meurer <benny@xfce.org>
@@ -205,9 +205,12 @@ cursor_changed (GtkTreeView * tv, PanelItemsDialog *pid)
     GtkTreeModel *model;
     GtkTreeIter iter;
     XfcePanelItemInfo *info;
-    
-    sel = gtk_tree_view_get_selection (tv);
-    gtk_tree_selection_get_selected (sel, &model, &iter);
+
+    if (!(sel = gtk_tree_view_get_selection (tv)))
+        return;
+
+    if (!gtk_tree_selection_get_selected (sel, &model, &iter))
+        return;
 
     gtk_tree_model_get (model, &iter, 0, &info, -1);
 }
@@ -256,12 +259,12 @@ render_text (GtkTreeViewColumn * col, GtkCellRenderer * cell,
         
         if (info->comment)
         {
-            g_snprintf (text, 512, "<b>%s</b>\n%s", info->display_name, 
+            g_snprintf (text, sizeof(text), "<b>%s</b>\n%s", info->display_name, 
                                     info->comment);
         }
         else
         {
-            g_snprintf (text, 512, "<b>%s</b>", info->display_name);
+            g_snprintf (text, sizeof(text), "<b>%s</b>", info->display_name);
         }
 
         g_object_set (cell, "markup", text, 
@@ -325,7 +328,8 @@ treeview_data_get (GtkWidget *widget, GdkDragContext *drag_context,
             return;
         }
         
-        gtk_tree_selection_get_selected (sel, &model, &iter);
+        if (!gtk_tree_selection_get_selected (sel, &model, &iter))
+            return;
 
         gtk_tree_model_get (model, &iter, 0, &info, -1);
 
@@ -653,8 +657,10 @@ add_items_dialog (GPtrArray *panels, GtkWidget *active_item)
     g_ptr_array_foreach (panels, (GFunc)item_dialog_opened, NULL);
 
     gtk_window_stick(GTK_WINDOW (dlg));
-    gtk_window_set_type_hint (GTK_WINDOW (dlg), GDK_WINDOW_TYPE_HINT_UTILITY);
 
+    gtk_widget_realize (dlg);
+    gdk_x11_window_set_user_time (GTK_WIDGET (dlg)->window,
+        gdk_x11_get_server_time (GTK_WIDGET (dlg)->window));
     xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (dlg));
     gtk_widget_show (dlg);
 
@@ -1201,16 +1207,8 @@ monitor_pressed (GtkToggleButton *tb, GdkEvent *ev, PanelManagerDialog *pmd)
 
                 if (mon == tb)
                 {
-                    XfceMonitor *xmon = panel_app_get_monitor(i);
                     gtk_toggle_button_set_active (mon, TRUE);
                     panel_set_monitor (pmd->panel, i);
-                    if (xmon->screen != gtk_widget_get_screen (pmd->dlg))
-                    {
-                        gtk_widget_hide (pmd->dlg);
-                        gtk_window_set_screen (GTK_WINDOW (pmd->dlg), 
-                                               xmon->screen);
-                        gtk_widget_show (pmd->dlg);
-                    }
                 }
                 else
                 {
@@ -1278,7 +1276,7 @@ add_monitor_selector (GtkBox *box, PanelManagerDialog *pmd)
                 gtk_widget_set_size_request (scroll, req.width, -1);
             }
 
-            g_snprintf (markup, 10, "<b>%d</b>", i + 1);
+            g_snprintf (markup, sizeof(markup), "<b>%d</b>", i + 1);
             
             ebox = gtk_event_box_new ();
             style = gtk_widget_get_style (ebox);
@@ -1384,7 +1382,7 @@ add_appearance_options (GtkBox *box, PanelManagerDialog *pmd)
     if (G_UNLIKELY (!composite_atom))
     {
         char text[16];
-        g_snprintf (text, 16, "_NET_WM_CM_S%d", 
+        g_snprintf (text, sizeof(text), "_NET_WM_CM_S%d", 
                     GDK_SCREEN_XNUMBER(gdk_screen_get_default()));
         composite_atom = 
             XInternAtom (GDK_DISPLAY (), text, False);
@@ -1431,7 +1429,7 @@ panel_selected (GtkComboBox * combo, PanelManagerDialog * pmd)
 {
     int n = gtk_combo_box_get_active (combo);
 
-    if (n == pmd->current)
+    if (n == pmd->current || n < 0)
         return;
 
     pmd->current = n;
@@ -1455,7 +1453,7 @@ add_panel (GtkWidget * w, PanelManagerDialog * pmd)
 
     panel_block_autohide (PANEL (g_ptr_array_index (pmd->panels, n)));
 
-    g_snprintf (name, 20, _("Panel %d"), pmd->panels->len);
+    g_snprintf (name, sizeof(name), _("Panel %d"), pmd->panels->len);
 
     gtk_combo_box_append_text (GTK_COMBO_BOX (pmd->panel_selector), name);
 
@@ -1491,7 +1489,7 @@ remove_panel (GtkWidget * w, PanelManagerDialog * pmd)
     {
         char name[20];
 
-        g_snprintf (name, 20, _("Panel %d"), i + 1);
+        g_snprintf (name, sizeof(name), _("Panel %d"), i + 1);
 
         gtk_combo_box_append_text (GTK_COMBO_BOX (pmd->panel_selector), name);
     }
@@ -1516,7 +1514,7 @@ create_panel_selector (PanelManagerDialog *pmd)
     {
         char name[20];
 
-        g_snprintf (name, 20, _("Panel %d"), i + 1);
+        g_snprintf (name, sizeof(name), _("Panel %d"), i + 1);
 
         gtk_combo_box_append_text (GTK_COMBO_BOX (pmd->panel_selector), name);
     }
@@ -1653,10 +1651,12 @@ panel_manager_dialog (GPtrArray *panels)
                       G_CALLBACK (panel_dialog_response), pmd);
     
     gtk_window_stick(GTK_WINDOW (pmd->dlg));
-    gtk_window_set_type_hint (GTK_WINDOW (pmd->dlg), 
-                              GDK_WINDOW_TYPE_HINT_UTILITY);
-
     xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (pmd->dlg));
+    gtk_widget_realize (pmd->dlg);
+
+    /* window needs to be realized */
+    gdk_x11_window_set_user_time (GTK_WIDGET (pmd->dlg)->window,
+        gdk_x11_get_server_time (GTK_WIDGET (pmd->dlg)->window));
     gtk_widget_show (pmd->dlg);
 
     panel_app_register_dialog (pmd->dlg);
