@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: clock.c 23501 2006-10-24 19:03:35Z jasper $
+/*  $Id: clock.c 25792 2007-06-10 09:48:20Z jasper $
  *
  *  Copyright © 2005 The Xfce Development Team
  *
@@ -61,6 +61,8 @@ typedef struct
     GtkTooltips *tips;
 
     int timeout_id;
+    int reschedule_id;
+    int screen_changed_id;
 
     /* Settings */
     int mode;
@@ -69,6 +71,7 @@ typedef struct
     gboolean secs;
 
     gboolean show_frame;
+    gint mday;
 }
 Clock;
 
@@ -93,22 +96,74 @@ static void clock_construct (XfcePanelPlugin *plugin);
 /* -------------------------------------------------------------------- *
  *                               Clock                                  *
  * -------------------------------------------------------------------- */
+ 
+static gboolean
+clock_reschedule_callback (Clock *clock)
+{
+    g_return_val_if_fail (clock->secs == FALSE, FALSE);
+    
+    /* update the clock every minute */
+    xfce_clock_set_interval (XFCE_CLOCK (clock->clock), 60 * 1000);
+    
+    return FALSE;
+}
+
+static void
+clock_reschedule_callback_destroy (Clock *clock)
+{
+    clock->reschedule_id = 0;
+}
+
+static void
+clock_reschedule (Clock *clock)
+{
+    time_t ticks;
+    struct tm *tm;
+    gint interval;
+    
+    /* stop running reschudule */
+    if (clock->reschedule_id)
+        g_source_remove (clock->reschedule_id);
+        
+    /* update every second if seconds are displayed or the led clock is used */
+    if (clock->secs || clock->mode == XFCE_CLOCK_LEDS)
+    {
+    	/* update every second */
+    	xfce_clock_set_interval (XFCE_CLOCK (clock->clock), 1000);
+    }
+    else
+    {
+        /* get the time */
+        time (&ticks);
+        tm = localtime (&ticks);
+    
+        /* get the interval up to the next minute */
+        interval = ((60 - tm->tm_sec) * 1000) + 500;
+        
+        /* set the new clock timeout (for this minute) */
+        xfce_clock_set_interval (XFCE_CLOCK (clock->clock), interval);
+        
+        /* start a new reschedule event */
+        clock->reschedule_id = 
+            g_timeout_add_full (G_PRIORITY_LOW, interval, (GSourceFunc)clock_reschedule_callback, 
+                                clock, (GDestroyNotify)clock_reschedule_callback_destroy);
+    }
+}
 
 static gboolean 
 clock_date_tooltip (Clock *clock)
 {
     time_t ticks;
     struct tm *tm;
-    static gint mday = -1;
     char date_s[255];
     char *utf8date = NULL;
 
     ticks = time (0);
     tm = localtime (&ticks);
 
-    if (mday != tm->tm_mday)
+    if (clock->mday != tm->tm_mday)
     {
-        mday = tm->tm_mday;
+        clock->mday = tm->tm_mday;
 
         /* Use format characters from strftime(3)
 	 * to get the proper string for your locale.
@@ -118,7 +173,7 @@ clock_date_tooltip (Clock *clock)
 	 * %B  : full month name
 	 * %Y  : four digit year
 	 */
-        strftime(date_s, 255, _("%A %d %B %Y"), tm);
+        strftime(date_s, sizeof(date_s), _("%A %d %B %Y"), tm);
 
         /* Conversion to utf8
          * Patch by Oliver M. Bolzer <oliver@fakeroot.net>
@@ -235,6 +290,11 @@ clock_free_data (XfcePanelPlugin *plugin, Clock *clock)
 
     if (dlg)
         gtk_widget_destroy (dlg);
+        
+    if (clock->reschedule_id)
+        g_source_remove (clock->reschedule_id);
+        
+    g_signal_handler_disconnect (plugin, clock->screen_changed_id);
     
     g_source_remove (clock->timeout_id);
     g_object_unref (G_OBJECT (clock->tips));
@@ -313,6 +373,28 @@ clock_write_rc_file (XfcePanelPlugin *plugin, Clock *clock)
     xfce_rc_close (rc);
 }
 
+static void
+clock_screen_changed (GtkWidget *plugin, GdkScreen *screen, 
+                      Clock *clock)
+{
+    /* return when the widget is not visible (when moving the plugin) */
+    if (GTK_IS_INVISIBLE (clock->clock) == FALSE)
+        return;
+
+    gtk_widget_destroy (clock->clock);
+
+    clock->clock = xfce_clock_new ();
+    gtk_widget_show (clock->clock);
+    gtk_container_add (GTK_CONTAINER (clock->frame), clock->clock);
+
+    clock->mday = -1;
+
+    clock_read_rc_file (clock->plugin, clock);
+    
+    /* set the clock timeout */
+    clock_reschedule (clock);
+}
+
 /* Create widgets and connect to signals */
 
 static void 
@@ -321,6 +403,7 @@ clock_construct (XfcePanelPlugin *plugin)
     Clock *clock = panel_slice_new0 (Clock);
 
     clock->plugin = plugin;
+    clock->reschedule_id = 0;
 
     g_signal_connect (plugin, "size-changed", 
                       G_CALLBACK (clock_set_size), clock);
@@ -334,6 +417,10 @@ clock_construct (XfcePanelPlugin *plugin)
     xfce_panel_plugin_menu_show_configure (plugin);
     g_signal_connect (plugin, "configure-plugin", 
                       G_CALLBACK (clock_properties_dialog), clock);
+                      
+    clock->screen_changed_id = 
+        g_signal_connect (plugin, "screen-changed", 
+                          G_CALLBACK (clock_screen_changed), clock);
 
     clock->frame = gtk_frame_new (NULL);
     gtk_widget_show (clock->frame);
@@ -343,18 +430,22 @@ clock_construct (XfcePanelPlugin *plugin)
     gtk_widget_show (clock->clock);
     gtk_container_add (GTK_CONTAINER (clock->frame), clock->clock);
 
-    xfce_clock_set_interval (XFCE_CLOCK (clock->clock), 1000);
-
     clock_read_rc_file (plugin, clock);
     
+    clock->mday = -1;
+
     clock->tips = gtk_tooltips_new ();
     g_object_ref (G_OBJECT (clock->tips));
     gtk_object_sink (GTK_OBJECT (clock->tips));
         
     clock_date_tooltip (clock);
-
+    
+    /* set the clock timeout */
+    clock_reschedule (clock);
+ 
+    /* update the tooltip every minute */
     clock->timeout_id = 
-        g_timeout_add (60000, (GSourceFunc) clock_date_tooltip, clock);
+        g_timeout_add (60 * 1000, (GSourceFunc) clock_date_tooltip, clock);
 }
 
 /* -------------------------------------------------------------------- *
@@ -362,7 +453,7 @@ clock_construct (XfcePanelPlugin *plugin)
  * -------------------------------------------------------------------- */
 
 static void
-clock_set_sensative (ClockDialog *cd)
+clock_set_sensitive (ClockDialog *cd)
 {
     if (cd->clock->mode == XFCE_CLOCK_ANALOG)
     {
@@ -397,7 +488,7 @@ clock_button_toggled (GtkWidget *cb, ClockDialog *cd)
 	xfce_clock_show_military (XFCE_CLOCK (cd->clock->clock),
 	                          active);
 	
-	clock_set_sensative (cd);
+	clock_set_sensitive (cd);
     }
     else if (cb == cd->cb_ampm)
     {
@@ -410,6 +501,9 @@ clock_button_toggled (GtkWidget *cb, ClockDialog *cd)
 	cd->clock->secs = active;
 	xfce_clock_show_secs (XFCE_CLOCK (cd->clock->clock),
 	                      active);
+	                      
+	/* set the clock timeout */
+        clock_reschedule (cd->clock);
     }
     
     clock_update_size (cd->clock, 
@@ -422,7 +516,9 @@ clock_mode_changed (GtkComboBox *cb, ClockDialog *cd)
     cd->clock->mode = gtk_combo_box_get_active(cb);
     xfce_clock_set_mode (XFCE_CLOCK (cd->clock->clock), cd->clock->mode);
     
-    clock_set_sensative (cd);
+    clock_reschedule (cd->clock);
+    
+    clock_set_sensitive (cd);
     
     clock_update_size (cd->clock, 
             xfce_panel_plugin_get_size (XFCE_PANEL_PLUGIN (cd->clock->plugin)));
@@ -457,6 +553,9 @@ clock_properties_dialog (XfcePanelPlugin *plugin, Clock *clock)
                 GTK_STOCK_CLOSE, GTK_RESPONSE_OK,
                 NULL);
 
+    gtk_window_set_screen (GTK_WINDOW (dlg), 
+                           gtk_widget_get_screen (GTK_WIDGET (plugin)));
+
     g_object_set_data (G_OBJECT (plugin), "dialog", dlg);
 
     gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
@@ -519,7 +618,7 @@ clock_properties_dialog (XfcePanelPlugin *plugin, Clock *clock)
     g_signal_connect (cb, "toggled",
             G_CALLBACK (clock_button_toggled), cd);
 
-    clock_set_sensative (cd);
+    clock_set_sensitive (cd);
 
     g_signal_connect (dlg, "response",
             G_CALLBACK (clock_dialog_response), cd);
