@@ -1,6 +1,6 @@
 /* vim: set expandtab ts=8 sw=4: */
 
-/*  $Id: launcher-dialog.c 23356 2006-10-11 19:10:40Z jasper $
+/*  $Id: launcher-dialog.c 25792 2007-06-10 09:48:20Z jasper $
  *
  *  Copyright © 2005 Jasper Huijsmans <jasper@xfce.org>
  *
@@ -1183,7 +1183,7 @@ tree_button_clicked (GtkWidget *b, LauncherDialog *ld)
         {
             char last[3];
 
-            g_snprintf (last, 3, "%d", ld->launcher->entries->len - 2);
+            g_snprintf (last, sizeof(last), "%d", ld->launcher->entries->len - 2);
             path = gtk_tree_path_new_from_string (last);
         }
         
